# -*- coding: utf-8 -*-

from flask import render_template, Blueprint, g, jsonify, redirect, abort, request, current_app, send_file, session,\
    url_for, make_response
import fmtapper_web.services as services
from utils import ApiErrorResponse, ApiDataResponse
from forms.tuner_forms import TunerForm
from forms.schedule_forms import ScheduleForm
from forms.cleanup_service_forms import CleanupServiceConfigForm
from fmtapper_web.tuner import Tuner
from fmtapper_web.schedule import Schedule, Interval, Schedules
from fmtapper_web import tuner_service_utils
from fmtapper_web import config_utils
import os
from decorators import login_required, api_admin_required, api_login_required
from werkzeug import secure_filename
from fmtapper_web.cleanup_service import CleanupService
from fmtapper_web.errors import ServiceMethodError
from wtforms import ValidationError
import auth_utils

mod = Blueprint('main', __name__)


@mod.route('/', methods=['GET'])
def index():
    return redirect('/tuners/')

@mod.route('/cleanup/', methods=['GET'])
@mod.route('/cleanup/logs/', methods=['GET'])
@mod.route('/tuners/<path:p>', methods=['GET'])
@mod.route('/tuners/', methods=['GET'])
@login_required
def tuners(p=None):
    return render_template('main/index.html', installation_id=config_utils.get_installation_id(),
                           name=session['username'])


@mod.route('/api/metrics', methods=['GET'])
@api_login_required
def get_metrics():
    metrics = services.get_system_info(getattr(g, '_database'))
    return jsonify(*ApiDataResponse(data=metrics))


@mod.route('/api/tuners', methods=['GET'])
@api_login_required
def get_tuners():
    tuner_list = services.get_tuners()
    return jsonify(*ApiDataResponse(data=tuner_list))


@mod.route('/api/tuners/<tuner_id>', methods=['GET'])
@api_login_required
def get_tuner(tuner_id):
    tuner = services.get_tuner(tuner_id)
    if tuner is None:
        return abort(404)
    else:
        return jsonify(tuner.to_json())


@mod.route('/api/tuners/<tuner_id>', methods=['PUT'])
@api_admin_required
def update_tuner(tuner_id):
    tuner_json = request.json
    tuner_json['id'] = tuner_id
    form = TunerForm.from_json(data=tuner_json)
    if form.validate():
        current_app.logger.debug('Tuner update request pass validation')
        tuner = Tuner.from_dict(tuner_json)
        services.update_tuner(tuner)
        return jsonify(*ApiDataResponse(data=tuner))
    else:
        current_app.logger.debug('Tuner update request failed validation')
        current_app.logger.debug(form.errors)
        return jsonify(*ApiErrorResponse(status=406, global_errors=None, field_errors=form.errors))


@mod.route('/api/tuners/<tuner_id>', methods=['POST'])
@api_admin_required
def apply_tuner(tuner_id):
    tuner_json = request.json
    tuner_json['id'] = tuner_id
    form = TunerForm.from_json(data=tuner_json)
    if form.validate():
        current_app.logger.debug('Tuner update request pass validation')
        tuner = Tuner.from_dict(tuner_json)
        services.apply_tuner(tuner)
        return jsonify(*ApiDataResponse(data=tuner))
    else:
        current_app.logger.debug('Tuner update request failed validation')
        current_app.logger.debug(form.errors)
        return jsonify(*ApiErrorResponse(status=406, global_errors=None, field_errors=form.errors))


@mod.route('/api/tuners/<tuner_id>/restart', methods=['POST'])
@api_admin_required
def restart_tuner(tuner_id):
    current_app.logger.debug('Tuner restart request')
    tuner = services.get_tuner(tuner_id)
    services.apply_tuner(tuner)
    return jsonify(*ApiDataResponse(data='Ok'))


@mod.route('/api/tuners', methods=['POST'])
@api_admin_required
def add_tuner():
    tuner_json = request.json
    form = TunerForm.from_json(data=tuner_json)
    if form.validate():
        current_app.logger.debug('Tuner create request pass validation')

        copy_schedules = False
        if 'copy_schedules' in tuner_json:
            copy_schedules = tuner_json['copy_schedules']
            del tuner_json['copy_schedules']

        tuner = Tuner.from_dict(tuner_json)
        if tuner.id in services.get_tuner_ids():
            err = {'id': 'Указанный Id уже используется.'}
            return jsonify(*ApiErrorResponse(status=406, global_errors=None, field_errors=err))

        services.create_tuner(tuner, copy_schedules)
        return jsonify(*ApiDataResponse(data=services.get_tuner(tuner.id)))
    else:
        current_app.logger.debug('Tuner create request failed validation')
        return jsonify(*ApiErrorResponse(status=406, global_errors=None, field_errors=form.errors))


@mod.route('/api/tuners/<tuner_id>', methods=['DELETE'])
@api_admin_required
def delete_tuner(tuner_id):
    password = request.args.get('password').encode('utf8')
    if password is None:
        jsonify(*ApiErrorResponse(global_errors=['Введите пароль'], status=406, field_errors=[]))
    elif auth_utils.get_user(session['username'], password) is None:
        return jsonify(*ApiErrorResponse(global_errors=['Неверный пароль'], status=406, field_errors=[]))
    else:
        services.delete_tuner(tuner_id)
        return jsonify(*ApiDataResponse(data="OK"))


@mod.route('/api/tuners/<tuner_id>/started', methods=['POST'])
@api_admin_required
def start_tuner(tuner_id):
    logs_dir = config_utils.get_tuner_logs_dir(tuner_id)
    services.update_ftp_service(config_utils.get_config_file_path(tuner_id), logs_dir)
    tuner_service_utils.start_tuner(tuner_id)
    return jsonify(*ApiDataResponse(data="OK"))


@mod.route('/api/tuners/<tuner_id>/started', methods=['DELETE'])
@api_admin_required
def stop_tuner(tuner_id):
    logs_dir = config_utils.get_tuner_logs_dir(tuner_id)
    services.update_ftp_service(config_utils.get_config_file_path(tuner_id), logs_dir, delete=True)
    tuner_service_utils.stop_tuner(tuner_id)
    services.apply_schedules(tuner_id)
    return jsonify(*ApiDataResponse(data="OK"))


@mod.route('/api/tuners/<tuner_id>/broadcasting', methods=['POST'])
@api_admin_required
def start_broadcast(tuner_id):
    if services.start_broadcast(tuner_id):
        return jsonify(*ApiDataResponse(data="OK"))
    else:
        return jsonify(*ApiErrorResponse(status=500, global_errors='Не удалось запустить трансляцию'))


@mod.route('/api/tuners/<tuner_id>/broadcasting', methods=['DELETE'])
@api_admin_required
def stop_broadcast(tuner_id):
    if services.stop_broadcast(tuner_id):
        return jsonify(*ApiDataResponse(data="OK"))
    else:
        return jsonify(*ApiErrorResponse(status=500, global_errors='Не удалось остановить трансляцию'))


@mod.route('/configs/<tuner_id>', methods=['GET'])
@api_login_required
def get_tuner_configs(tuner_id):
    arch_path = services.archive_tuner_configs(tuner_id)
    return send_file(arch_path, as_attachment=True)


def allowed_file(filename):
    return '.' in filename and filename.rsplit('.', 1)[1] == 'zip'


@mod.route('/api/configs/<tuner_id>', methods=['POST'])
@api_admin_required
def upload_tuner_configs(tuner_id):
    uploaded = request.files['archive']
    if uploaded and allowed_file(uploaded.filename):
        filename = secure_filename(uploaded.filename)
        file_path = os.path.join(current_app.config['UPLOAD_FOLDER'], filename)
        uploaded.save(file_path)

        if tuner_id == "add_new_tuner":
            tuner = services.add_new_tuner_from_archive(file_path)
            if tuner is not None:
                return jsonify(tuner.to_json())
        else:
            is_applied = services.apply_archived_tuner_configs(tuner_id, file_path)
            if is_applied:
                return jsonify(*ApiErrorResponse(status=200, global_errors=''))
            else:
                return jsonify(*ApiErrorResponse(status=406, global_errors='Указан неверный файл конфигурации'))

    return jsonify(*ApiErrorResponse(status=406, global_errors='Указан неверный файл конфигурации'))


@mod.route('/api/devices', methods=['GET'])
@api_login_required
def get_devices():
	try:
		return jsonify(*ApiDataResponse(data=services.get_audio_devices()))
	except:
		return jsonify(*ApiDataResponse(data=''))


@mod.route('/api/tuners/<tuner_id>/schedules', methods=['GET'])
@api_login_required
def get_schedules(tuner_id):
    return jsonify(*ApiDataResponse(data={'schedules': services.get_schedules(tuner_id)}))


@mod.route('/api/tuners/<tuner_id>/schedules/<sch_id>', methods=['GET'])
@api_login_required
def get_schedule(tuner_id, sch_id):
    schedule = services.get_schedule(tuner_id, sch_id)
    if schedule:
        return jsonify(schedule.to_json())
    else:
        abort(404)


@mod.route('/api/tuners/<tuner_id>/schedules/<sch_id>', methods=['POST'])
@api_login_required
def apply_schedule(tuner_id, sch_id):
    schedule = services.get_schedule(tuner_id, sch_id)
    services.apply_schedule(schedule, tuner_id)
    if schedule:
        return jsonify(schedule.to_json())
    else:
        abort(404)


@mod.route('/api/tuners/<tuner_id>/schedules/<sch_id>', methods=['PUT'])
@api_admin_required
def update_schedule(tuner_id, sch_id):
    schedule_json = request.json
    schedule_json['id'] = sch_id
    form = ScheduleForm.from_json(data=schedule_json)
    if form.validate():
        current_app.logger.debug('Schedule update request pass validation')
        schedule = Schedule.from_dict(schedule_json)
        schedule.intervals = sorted(schedule.intervals, Interval.comparator)
        services.update_schedule(tuner_id, schedule)
        return jsonify(*ApiDataResponse(data=schedule))
    else:
        current_app.logger.debug('Schedule update request failed validation')
        return jsonify(*ApiErrorResponse(status=406, global_errors=None, field_errors=form.errors))


@mod.route('/api/tuners/<tuner_id>/schedules', methods=['POST'])
@api_admin_required
def create_schedule(tuner_id):
    schedule_json = request.json
    form = ScheduleForm.from_json(data=schedule_json)
    if form.validate():
        current_app.logger.debug('Schedule create request pass validation')
        schedule = Schedule.from_dict(schedule_json)
        schedule.intervals = sorted(schedule.intervals, Interval.comparator)
        services.create_schedule(tuner_id, schedule)
        return jsonify(*ApiDataResponse(data=schedule))
    else:
        current_app.logger.debug('Schedule create request failed validation')
        return jsonify(*ApiErrorResponse(status=406, global_errors=None, field_errors=form.errors))


@mod.route('/api/tuners/<tuner_id>/schedules/<sch_id>', methods=['DELETE'])
@api_admin_required
def delete_schedule(tuner_id, sch_id):
    services.delete_schedule(tuner_id, sch_id)
    dic = {'is_applied': Schedules.is_schedules_applied(tuner_id)}
    return jsonify(*ApiDataResponse(data=dic))


@mod.route('/api/tuners/<tuner_id>/audio', methods=['GET'])
@api_login_required
def get_audio(tuner_id):
    return jsonify(*ApiDataResponse(data=services.get_audio_files(tuner_id)))


@mod.route('/api/tuners/<tuner_id>/logs', methods=['GET'])
@api_login_required
def get_logs(tuner_id):
    return jsonify(*ApiDataResponse(data=services.get_log_files(tuner_id)))


@mod.route('/get-audio/<tuner_id>/<path:name>')
@api_login_required
def get_audio_file(tuner_id, name):
    if ('' + name).find('..') >= 0:
        abort(404)
    file_path = os.path.join(services.get_tuner_audio_dir(tuner_id), name)
    if not os.path.isfile(file_path):
        abort(404)
    rv = make_response(send_file(file_path, as_attachment=True))
    rv.headers.add('Accept-Ranges', 'bytes')
    return rv


@mod.route('/get-log/<tuner_id>/<path:name>')
@api_login_required
def get_log_file(tuner_id, name):
    if ('' + name).find('..') >= 0:
        abort(404)
    file_path = os.path.join(config_utils.get_tuner_logs_dir(tuner_id), name)
    if not os.path.isfile(file_path):
        abort(404)
    return send_file(file_path, as_attachment=True)


@mod.route('/login', methods=['POST', 'GET'])
def login():
    logger = current_app.logger
    if request.method == 'POST':
        username = request.form['username']
        password = request.form['password']
        if username is None or len(username.strip()) == 0:
            return render_template('login.html', error=u'Укажите имя пользователя')
        if auth_utils.get_user(username, password) is None:
            logger.debug('Wrong username/password %s', username)
            return render_template('login.html', error=u'Неверное имя пользователя или пароль', username=username)
        else:
            session['username'] = username
            logger.debug('User logged in: %s', username)
            return redirect(url_for('.tuners'))
    else:
        if 'username' in session:
            return redirect(url_for('.tuners'))
        else:
            return render_template('login.html')


@mod.route('/logout')
def logout():
    session.pop('username', None)
    return redirect(url_for('.login'))


@mod.route('/api/cleanup_service', methods=['GET'])
@api_login_required
def get_cleanup_service():
    return jsonify(services.get_clenup_service().to_json())


@mod.route('/api/cleanup_service', methods=['PUT'])
@api_admin_required
def update_cleanup_service():
    cleanup_json = request.json
    form = CleanupServiceConfigForm.from_json(data=cleanup_json)
    if form.validate():
        current_app.logger.debug('Cleanup service update request pass validation')
        cleanup_service = CleanupService(**cleanup_json)
        services.update_cleanup_service(cleanup_service)
        return jsonify(*ApiDataResponse(data=cleanup_service))
    else:
        current_app.logger.debug('Cleanup service update request failed validation')
        return jsonify(*ApiErrorResponse(status=406, global_errors=None, field_errors=form.errors))


@mod.errorhandler(ServiceMethodError)
def handle_service_error(error):
    return jsonify(*ApiErrorResponse(status=500, global_errors=error))


@mod.errorhandler(ValidationError)
def handle_service_error(error):
    return jsonify(*ApiErrorResponse(status=406, global_errors=error))


@mod.route('/api/cleanup_service/logs', methods=['GET'])
@api_login_required
def get_cleanup_service_logs():
    return jsonify(*ApiDataResponse(data=services.get_cleanup_service_log_files()))


@mod.route('/get-cleanup-log/<name>')
@api_login_required
def get_cleanup_service_log_file(name):
    fname = os.path.basename(name)
    if not fname.startswith(config_utils.get_cleanup_service_log_pattern()):
        abort(404)
    file_path = os.path.join(config_utils.get_cleanup_service_logs_dir(), fname)
    if not os.path.isfile(file_path):
        abort(404)
    return send_file(file_path, as_attachment=True)


@mod.route('/api/tuners_status', methods=['GET'])
@api_login_required
def get_tuners_status():
    return jsonify(*ApiDataResponse(data=services.get_tuners_status()))


@mod.route('/api/tuners_status/<tuner_id>', methods=['GET'])
@api_login_required
def get_tuner_status(tuner_id):
    return jsonify(*ApiDataResponse(data=services.get_tuner_status(tuner_id)))


@mod.route('/api/current_server_time', methods=['GET'])
@api_login_required
def get_current_server_time():
    return jsonify(*ApiDataResponse(data=config_utils.get_current_time()))


@mod.route('/api/fmtapper_ver', methods=['GET'])
@api_login_required
def get_fmtapper_ver():
    return jsonify(*ApiDataResponse(data=services.get_fmtapper_ver()))


@mod.route('/api/fmtapper_features', methods=['GET'])
@api_login_required
def get_fmtapper_features():
    return jsonify(*ApiDataResponse(data=services.get_fmtapper_features()))
